function evalSegmentation()
% This script is used to evaluate actor-action semantic segmentation.
% Please organize your generated results in the same format as ground 
% truth color images.

% Path to videoset.csv
csvPath = '/PATH/TO/A2D/videoset.csv';

% Path to ground truth color images.
gtPath = '/PATH/TO/A2D/Annotations/col';

% Path to YOUR generated color PNG image results.
rsltPath = '/PATH/TO/YOUR/RESULTS';


%% Evaluation

VS = readInfo(csvPath);

% Color Map for labels
CMAP = initialCMAP();
labelMap = CMAP(:,1);
validMap = logical(CMAP(:,2));
codeMap = CMAP(:,3)*1000000 + CMAP(:,4)*1000 + CMAP(:,5);

allJoint = zeros(size(labelMap));
onJoint = zeros(size(labelMap));

actorList = unique(floor(labelMap/10));
allActor = zeros(size(actorList));
onActor = zeros(size(actorList));

actionList = unique(mod(labelMap,10));
allAction = zeros(size(actionList));
onAction = zeros(size(actionList));

for i=1:length(VS.vid)
    if VS.test(i) == 1
        
        frameList = dir(fullfile(gtPath, VS.vid{i}, '*.png'));
        for j=1:length(frameList)
            gtIm = imread(fullfile(gtPath, VS.vid{i}, frameList(j).name));
            try
                rsltIm = imread(fullfile(rsltPath, VS.vid{i}, frameList(j).name));
            catch err
                error('Unable to read %s!', fullfile(rsltPath, frameList(j).name));
            end
            
            gtIm = double(gtIm);
            rsltIm = double(rsltIm);
            
            codeGtList = gtIm(:,:,1)*1000000 + gtIm(:,:,2)*1000 + gtIm(:,:,3);
            codeGtList = codeGtList(:);
            [~, idx] = ismember(codeGtList, codeMap);
            labelGtList = labelMap(idx);
            
            codeRsltList = rsltIm(:,:,1)*1000000 + rsltIm(:,:,2)*1000 + rsltIm(:,:,3);
            codeRsltList = codeRsltList(:);
            [~, idx] = ismember(codeRsltList, codeMap);
            labelRsltList = labelMap(idx);
            
            % Joint Label
            allJoint = allJoint + histc(labelGtList, labelMap);
            idx = (labelGtList==labelRsltList);
            onJoint = onJoint + histc(labelGtList(idx), labelMap);
            
            % Actor Label
            tmpLabelGtList = floor(labelGtList/10);
            tmpLabelRsltList = floor(labelRsltList/10);
            allActor = allActor + histc(tmpLabelGtList, actorList);
            idx = (tmpLabelGtList==tmpLabelRsltList);
            onActor = onActor + histc(tmpLabelGtList(idx), actorList);
            
            % Action Label
            tmpLabelGtList = mod(labelGtList,10);
            tmpLabelRsltList = mod(labelRsltList,10);
            allAction = allAction + histc(tmpLabelGtList, actionList);
            idx = (tmpLabelGtList==tmpLabelRsltList);
            onAction = onAction + histc(tmpLabelGtList(idx), actionList);
            
        end
        
    end
    
    
    if mod(i,100) == 0
        fprintf('processed %f of all videos ... \n', i/length(VS.vid));
    end
end


fprintf('Actor-Action Joint Label\n');
allJoint = allJoint(validMap);
onJoint = onJoint(validMap);
jointPerClass = onJoint./allJoint;
jointGlobal = sum(onJoint)/sum(allJoint);
fprintf('    Average Per-Class Accuracy: %f \n', mean(jointPerClass));
fprintf('    Global Pixel Accuracy: %f \n', jointGlobal);

fprintf('Actor Label\n');
actorPerClass = onActor./allActor;
fprintf('    Average Per-Class Accuracy: %f \n', mean(actorPerClass));

fprintf('Action Label\n');
actionPerClass = onAction./allAction;
fprintf('    Average Per-Class Accuracy: %f \n', mean(actionPerClass));

fprintf('Done.\n');

function VS = readInfo(csvPath)
% csv file structure:
% vid, label, timestart, timeend, height, width, frames, anno, test
% vid: 11-digit YouTube video id.
% label: single video-level label (e.g. 17 (adult-running)).
% timestart: start time to cut for a clip out of a video.
% timeend: end time for the clip.
% height: resized height.
% width: resized width.
% frames: number of frames of the cut clip.
% anno: number of VALID annotation frames.
% test: 0 - train; 1 - test.

fid = fopen(csvPath, 'r');
C = textscan(fid, '%q%d%q%q%d%d%d%d%d', 'delimiter', ',');
fclose(fid);

VS.vid = C{1};
VS.label = C{2};
VS.timeStart = C{3};
VS.timeEnd = C{4};
VS.height = C{5};
VS.width = C{6};
VS.frames = C{7};
VS.anno = C{8};
VS.test = C{9};

function CMAP = initialCMAP()
% This is the label id & color map.
%   Columns: id, valid, R, G, B

CMAP = [0,1,0,0,0;
    11,1,52,1,1;12,1,103,1,1;13,1,154,1,1;14,0,205,1,1;15,1,255,1,1;
    16,1,255,51,51;17,1,255,103,103;18,1,255,154,154;19,1,255,205,205;
    21,1,52,46,1;22,1,103,92,1;23,0,154,138,1;24,0,205,184,1;25,0,255,230,1;
    26,1,255,235,51;27,0,255,240,103;28,1,255,245,154;29,1,255,250,205;
    31,0,11,52,1;32,0,21,103,1;33,0,31,154,1;34,1,41,205,1;35,1,52,255,1;
    36,1,92,255,51;37,0,133,255,103;38,0,174,255,154;39,1,215,255,205;
    41,1,1,52,36;42,0,1,103,72;43,1,1,154,108;44,1,1,205,143;45,1,1,255,179;
    46,1,51,255,194;47,0,103,255,210;48,1,154,255,225;49,1,205,255,240;
    51,0,1,21,52;52,0,1,41,103;53,0,1,62,154;54,1,1,82,205;55,1,1,103,255;
    56,1,51,133,255;57,1,103,164,255;58,0,154,194,255;59,1,205,225,255;
    61,1,26,1,52;62,0,52,1,103;63,1,77,1,154;64,0,103,1,205;65,1,128,1,255;
    66,1,154,51,255;67,1,179,103,255;68,1,205,154,255;69,1,230,205,255;
    71,0,52,1,31;72,1,103,1,62;73,1,154,1,92;74,0,205,1,123;75,1,255,1,153;
    76,1,255,51,174;77,1,255,103,194;78,1,255,154,215;79,1,255,205,235;
    ];
